@@ -17,16 +17,10 @@
  */
 
 #include "goxel.h"
+#include <limits.h>
 
 static const int N = BLOCK_SIZE;
 
-#define DATA_AT(d, x, y, z) (d->voxels[x + y * N + z * N * N])
-
-#define BLOCK_ITER_INSIDE(x, y, z) \
-    for (z = 1; z < N - 1; z++) \
-        for (y = 1; y < N - 1; y++) \
-            for (x = 1; x < N - 1; x++)
-
 // Marching cube data.
 static const int MC_EDGE_TABLE[256];
 static const int8_t MC_TRI_TABLE[256][16];
@@ -69,16 +63,14 @@ static int mc_compute(const int neighboors[8],
     return nb_tri;
 }
 
-static vec3b_t mc_interp_pos(const mc_vert_t *vert)
+static void mc_interp_pos(const mc_vert_t *vert, uint8_t out[3])
 {
     int i;
-    vec3b_t ret = vec3b_zero;
-    vec3b_t p0 = VERTICES_POSITIONS[vert->v0];
-    vec3b_t p1 = VERTICES_POSITIONS[vert->v1];
+    const int *p0 = VERTICES_POSITIONS[vert->v0];
+    const int *p1 = VERTICES_POSITIONS[vert->v1];
     const float mu = vert->mu;
     for (i = 0; i < 3; i++)
-        ret.v[i] = (p0.v[i] * (1 - mu) + p1.v[i] * mu) * MC_VOXEL_SUB_POS;
-    return ret;
+        out[i] = (p0[i] * (1 - mu) + p1[i] * mu) * MC_VOXEL_SUB_POS;
 }
 
 static vec3_t mc_interp_normal(const mc_vert_t *vert, int normals[8][3])
@@ -94,48 +86,95 @@ static vec3_t mc_interp_normal(const mc_vert_t *vert, int normals[8][3])
     return ret;
 }
 
-int block_generate_vertices_mc(const block_data_t *data, int effects,
-                               voxel_vertex_t *out)
+int mesh_generate_vertices_mc(const mesh_t *mesh, const int block_pos[3],
+                              int effects, voxel_vertex_t *out)
 {
     int i, vi, x, y, z, v, w, vx, vy, vz, wx, wy, wz, nb_tri, nb_tri_tot = 0;
     int a, sum_a;
     vec3_t n;
-    uvec4b_t color;
+    uint8_t color[4], tmp[4];
     int colorbest;
     bool use_max_color;
+    uint8_t *data;
 
     int densities[8];
     int normals[8][3];
     int d, k = 2;
+    int p[3], s[3];
+    int rect[2][3] = {{INT_MAX, INT_MAX, INT_MAX},
+                      {INT_MIN, INT_MIN, INT_MIN}};
 
     mc_vert_t tri[5][3];
     if (!(effects & EFFECT_FLAT)) k = 8;
 
+    // To speed things up we first get the voxel cube around the block.
+    data = malloc((N + 2) * (N + 2) * (N + 2) * 4);
+    p[0] = block_pos[0] - 1;
+    p[1] = block_pos[1] - 1;
+    p[2] = block_pos[2] - 1;
+    s[0] = N + 2;
+    s[1] = N + 2;
+    s[2] = N + 2;
+    mesh_read(mesh, p, s, data);
+
+#define get_at(d, x, y, z, out) do { \
+    memcpy(out, &data[( \
+                (x + 1) + \
+                (y + 1) * (N + 2) + \
+                (z + 1) * (N + 2) * (N + 2)) * 4], 4); \
+} while (0)
+
+    // Get the smallest rect we need to consider.
+    // XXX: can we measure how much we gain with that?
+    for (z = -1; z < N + 1; z++)
+    for (y = -1; y < N + 1; y++)
+    for (x = -1; x < N + 1; x++) {
+        get_at(data, x, y, z, tmp);
+        if (tmp[3]) {
+            rect[0][0] = min(rect[0][0], x - 2);
+            rect[0][1] = min(rect[0][1], y - 2);
+            rect[0][2] = min(rect[0][2], z - 2);
+            rect[1][0] = max(rect[1][0], x + 2);
+            rect[1][1] = max(rect[1][1], y + 2);
+            rect[1][2] = max(rect[1][2], z + 2);
+        }
+    }
+    rect[0][0] = max(rect[0][0], 0);
+    rect[0][1] = max(rect[0][1], 0);
+    rect[0][2] = max(rect[0][2], 0);
+    rect[1][0] = min(rect[1][0], N);
+    rect[1][1] = min(rect[1][1], N);
+    rect[1][2] = min(rect[1][2], N);
+
     // Add up the contribution of each voxel to the vertices values.
-    BLOCK_ITER_INSIDE(x, y, z) {
+    for (z = rect[0][2]; z < rect[1][2]; z++)
+    for (y = rect[0][1]; y < rect[1][1]; y++)
+    for (x = rect[0][0]; x < rect[1][0]; x++) {
         memset(densities, 0, sizeof(densities));
         memset(normals, 0, sizeof(normals));
         n = vec3_zero;
-        color = DATA_AT(data, x, y, z);
-        use_max_color = (color.a == 0);
+        get_at(data, x, y, z, color);
+        use_max_color = (color[3] == 0);
         colorbest = 8;
         sum_a = 0;
         for (v = 0; v < 8; v++) {
 
-            vx = x + VERTICES_POSITIONS[v].x;
-            vy = y + VERTICES_POSITIONS[v].y;
-            vz = z + VERTICES_POSITIONS[v].z;
+            vx = x + VERTICES_POSITIONS[v][0];
+            vy = y + VERTICES_POSITIONS[v][1];
+            vz = z + VERTICES_POSITIONS[v][2];
 
             for (w = 0; w < 8; w++) {
-                wx = vx + VERTICES_POSITIONS[w].x - 1;
-                wy = vy + VERTICES_POSITIONS[w].y - 1;
-                wz = vz + VERTICES_POSITIONS[w].z - 1;
-                a = DATA_AT(data, wx, wy, wz).a;
+                wx = vx + VERTICES_POSITIONS[w][0] - 1;
+                wy = vy + VERTICES_POSITIONS[w][1] - 1;
+                wz = vz + VERTICES_POSITIONS[w][2] - 1;
+
+                get_at(data, wx, wy, wz, tmp);
+                a = tmp[3];
 
                 if (use_max_color && a) {
                     d = abs(x - wx) + abs(y - wy) + abs(z - wz);
                     if (d < colorbest) {
-                        color = DATA_AT(data, wx, wy, wz);
+                        get_at(data, wx, wy, wz, color);
                         colorbest = d;
                     }
                 }
@@ -144,14 +183,14 @@ int block_generate_vertices_mc(const block_data_t *data, int effects,
                 densities[v] += a;
 
                 if (!(effects & EFFECT_FLAT)) {
-                    normals[v][0] -= a * (2 * VERTICES_POSITIONS[w].x - 1);
-                    normals[v][1] -= a * (2 * VERTICES_POSITIONS[w].y - 1);
-                    normals[v][2] -= a * (2 * VERTICES_POSITIONS[w].z - 1);
+                    normals[v][0] -= a * (2 * VERTICES_POSITIONS[w][0] - 1);
+                    normals[v][1] -= a * (2 * VERTICES_POSITIONS[w][1] - 1);
+                    normals[v][2] -= a * (2 * VERTICES_POSITIONS[w][2] - 1);
                 }
                 else {
-                    n.x -= a * (2 * VERTICES_POSITIONS[w].x - 1);
-                    n.y -= a * (2 * VERTICES_POSITIONS[w].y - 1);
-                    n.z -= a * (2 * VERTICES_POSITIONS[w].z - 1);
+                    n.x -= a * (2 * VERTICES_POSITIONS[w][0] - 1);
+                    n.y -= a * (2 * VERTICES_POSITIONS[w][1] - 1);
+                    n.z -= a * (2 * VERTICES_POSITIONS[w][2] - 1);
                 }
             }
             densities[v] /= k;
@@ -163,20 +202,25 @@ int block_generate_vertices_mc(const block_data_t *data, int effects,
         for (i = 0; i < nb_tri; i++) {
             for (v = 0; v < 3; v++) {
                 vi = (nb_tri_tot + i) * 3 + v;
-                out[vi].color = color;
-                out[vi].color.a = 255;
-                out[vi].pos = mc_interp_pos(&tri[i][v]);
+                memcpy(out[vi].color, color, sizeof(color));
+                out[vi].color[3] = 255;
+                mc_interp_pos(&tri[i][v], out[vi].pos);
                 if (!(effects & EFFECT_FLAT))
                     n = mc_interp_normal(&tri[i][v], normals);
-                out[vi].normal = vec3b(n.x * 126, n.y * 126, n.z * 126);
-                vec3b_iaddk(&out[vi].pos, vec3b(x, y, z), MC_VOXEL_SUB_POS);
+                out[vi].normal[0] = n.x * 126;
+                out[vi].normal[1] = n.y * 126;
+                out[vi].normal[2] = n.z * 126;
+                out[vi].pos[0] += x * MC_VOXEL_SUB_POS;
+                out[vi].pos[1] += y * MC_VOXEL_SUB_POS;
+                out[vi].pos[2] += z * MC_VOXEL_SUB_POS;
                 // XXX: this shouldn't matter.
-                out[vi].bshadow_uv = uvec2b(0, 0);
-                out[vi].bump_uv = uvec2b(0, 0);
+                memset(out[vi].bshadow_uv, 0, sizeof(out[vi].bshadow_uv));
+                memset(out[vi].bump_uv, 0, sizeof(out[vi].bump_uv));
             }
         }
         nb_tri_tot += nb_tri;
     }
+    free(data);
     return nb_tri_tot;
 }
 
